#!/usr/bin/env python

from distutils.core import setup
import os
import glob

name = "python-amarok"
version = "0.1.0"

setup(name=name,
	version=version,
	description="A complete proxy class for controlling amaroK from Python",
	long_description="This module contains a nearly feature-complete class that can be used to control amaroK from amaroK scripts.",
	author="Manuel Amador (Rudd-O)",
	author_email="dragonfear@gmail.com",
	url="http://www.usm.edu.ec/~amadorm/software/",
	py_modules=["amarok","pydcopwrapper"],
	)
